'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _get2 = require('../utils/get');

var _get3 = _interopRequireDefault(_get2);

var _identity = require('lodash/identity');

var _identity2 = _interopRequireDefault(_identity);

var _icepick = require('icepick');

var _icepick2 = _interopRequireDefault(_icepick);

var _getValue = require('../utils/get-value');

var _getValue2 = _interopRequireDefault(_getValue);

var _isMulti = require('../utils/is-multi');

var _isMulti2 = _interopRequireDefault(_isMulti);

var _actionTypes = require('../action-types');

var _actionTypes2 = _interopRequireDefault(_actionTypes);

var _mapValues = require('../utils/map-values');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _track = require('../utils/track');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var defaultStrategies = {
  get: _get3.default,
  getValue: _getValue2.default,
  splice: _icepick2.default.splice,
  merge: _icepick2.default.merge,
  remove: _icepick2.default.dissoc
};

function createModelActions() {
  var s = arguments.length <= 0 || arguments[0] === undefined ? defaultStrategies : arguments[0];

  var change = function change(model, value) {
    var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

    // option defaults
    var changeOptions = _extends({
      silent: false,
      multi: (0, _isMulti2.default)(model)
    }, options);

    return _extends({
      type: _actionTypes2.default.CHANGE,
      model: model,
      value: s.getValue(value)
    }, changeOptions);
  };

  var xor = function xor(model, item) {
    var iteratee = arguments.length <= 2 || arguments[2] === undefined ? function (value) {
      return value === item;
    } : arguments[2];
    return function (dispatch, getState) {
      var state = s.get(getState(), model, []);
      var stateWithoutItem = state.filter(function (stateItem) {
        return !iteratee(stateItem);
      });
      var value = state.length === stateWithoutItem.length ? [].concat(_toConsumableArray(state), [item]) : stateWithoutItem;

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: value
      });
    };
  };

  var push = function push(model) {
    var item = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];
    return function (dispatch, getState) {
      var collection = s.get(getState(), model);
      var value = [].concat(_toConsumableArray(collection || []), [item]);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: value
      });
    };
  };

  var toggle = function toggle(model) {
    return function (dispatch, getState) {
      var value = !s.get(getState(), model);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: value
      });
    };
  };

  var filter = function filter(model) {
    var iteratee = arguments.length <= 1 || arguments[1] === undefined ? _identity2.default : arguments[1];
    return function (dispatch, getState) {
      var collection = s.get(getState(), model);
      var value = collection.filter(iteratee);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: value
      });
    };
  };

  var reset = function reset(model) {
    return {
      type: _actionTypes2.default.RESET,
      model: model
    };
  };

  var map = function map(model) {
    var iteratee = arguments.length <= 1 || arguments[1] === undefined ? _identity2.default : arguments[1];
    return function (dispatch, getState) {
      var collection = s.get(getState(), model, []);
      var value = collection.map(iteratee);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: value
      });
    };
  };

  var remove = function remove(model, index) {
    return function (dispatch, getState) {
      var collection = s.get(getState(), model, []);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: s.splice(collection, index, 1),
        removeKeys: [index]
      });
    };
  };

  var move = function move(model, indexFrom, indexTo) {
    return function (dispatch, getState) {
      var collection = s.get(getState(), model, []);

      if (indexFrom >= collection.length || indexTo >= collection.length) {
        throw new Error('Error moving array item: invalid bounds ' + indexFrom + ', ' + indexTo);
      }

      var item = collection[indexFrom];
      var removed = s.splice(collection, indexFrom, 1);
      var inserted = s.splice(removed, indexTo, 0, item);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: inserted
      });
    };
  };

  var merge = function merge(model, values) {
    return function (dispatch, getState) {
      var value = s.get(getState(), model, {});

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: s.merge(value, values)
      });
    };
  };

  var omit = function omit(model) {
    var props = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];
    return function (dispatch, getState) {
      var value = s.get(getState(), model, {});

      var propsArray = typeof props === 'string' ? [props] : props;

      var newValue = propsArray.reduce(function (acc, prop) {
        return s.remove(acc, prop);
      }, value);

      dispatch({
        type: _actionTypes2.default.CHANGE,
        model: model,
        value: newValue,
        removeKeys: propsArray
      });
    };
  };

  var load = function load(model, values) {
    return change(model, values, {
      silent: true
    });
  };

  return (0, _mapValues2.default)({
    change: change,
    filter: filter,
    map: map,
    merge: merge,
    push: push,
    remove: remove,
    move: move,
    reset: reset,
    toggle: toggle,
    xor: xor,
    load: load,
    omit: omit
  }, _track.trackable);
}

exports.default = createModelActions();