'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = changeActionReducer;

var _actionTypes = require('../../action-types');

var _actionTypes2 = _interopRequireDefault(_actionTypes);

var _icepick = require('icepick');

var _icepick2 = _interopRequireDefault(_icepick);

var _get = require('../../utils/get');

var _get2 = _interopRequireDefault(_get);

var _shallowEqual = require('../../utils/shallow-equal');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _compact = require('lodash/compact');

var _compact2 = _interopRequireDefault(_compact);

var _mapValues = require('../../utils/map-values');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _formReducer = require('../form-reducer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function updateFieldValue(field, action) {
  var method = void 0;
  var value = action.value;
  var removeKeys = action.removeKeys;
  var silent = action.silent;


  if ((0, _shallowEqual2.default)(field.value, value)) return field;

  if (silent) return _icepick2.default.set(field, 'value', value);

  if (removeKeys) {
    var _ret = function () {
      var result = [];

      Object.keys(field).forEach(function (key) {
        if (!!~removeKeys.indexOf(+key) || key === '$form') return;

        result[key] = field[key];
      });

      return {
        v: _extends({}, (0, _compact2.default)(result), { $form: field.$form })
      };
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
  }

  if (Array.isArray(value)) {
    method = function method(val, iter) {
      return Array.prototype.map.call(val, iter).filter(function (a) {
        return !!a;
      });
    };
  } else if ((0, _isPlainObject2.default)(value)) {
    method = function method(val, iter) {
      return (0, _mapValues2.default)(val, iter);
    };
  } else {
    return _icepick2.default.merge(field, {
      value: value,
      pristine: false,
      validated: false,
      retouched: field.submitted ? true : field.retouched
    });
  }

  var updatedField = method(value, function (subValue, index) {
    var subField = field[index];

    if (subField) {
      if (Object.hasOwnProperty.call(subField, '$form')) {
        return updateFieldValue(subField, subValue);
      }

      if ((0, _shallowEqual2.default)(subValue, subField.value)) {
        return subField;
      }

      return _icepick2.default.merge(subField, {
        value: subValue,
        pristine: false,
        validated: false,
        retouched: field.submitted ? true : subField.retouched
      });
    }

    // Subfield did not exist or was removed
    return false;
  });

  var dirtyFormState = _icepick2.default.merge(field.$form || _formReducer.initialFieldState, {
    pristine: false,
    validated: false,
    retouched: field.submitted ? true : (field.$form || _formReducer.initialFieldState).retouched
  });

  return _icepick2.default.set(updatedField, '$form', _icepick2.default.set(dirtyFormState, 'value', value));
}

function changeActionReducer(state, action, localPath) {
  if (action.type !== _actionTypes2.default.CHANGE) return state;

  var field = (0, _get2.default)(state, localPath, _formReducer.initialFieldState);

  var updatedField = updateFieldValue(field, action);

  if (!localPath.length) return updatedField;

  return _icepick2.default.setIn(state, localPath, updatedField);
}